@@ -26,6 +26,8 @@ CACHE_TYPES = {
     'unread_request_count': 'fruc-%d',
     'read_requests': 'frr-%d',
     'rejected_requests': 'frj-%d',
+    'unrejected_requests': 'frur-%d',
+    'unrejected_request_count': 'frurc-%d',
 }
 
 BUST_CACHES = {
@@ -38,7 +40,10 @@ BUST_CACHES = {
         'unread_request_count',
         'read_requests',
         'rejected_requests',
+        'unrejected_requests',
+        'unrejected_request_count',
     ],
+    'sent_requests': ['sent_requests'],
 }
 
 
@@ -103,7 +108,13 @@ class FriendshipRequest(models.Model):
             to_user=self.from_user
         ).delete()
 
+        # Bust requests cache - request is deleted
         bust_cache('requests', self.to_user.pk)
+        bust_cache('sent_requests', self.from_user.pk)
+        # Bust reverse requests cache - reverse request might be deleted
+        bust_cache('requests', self.from_user.pk)
+        bust_cache('sent_requests', self.to_user.pk)
+        # Bust friends cache - new friends added
         bust_cache('friends', self.to_user.pk)
         bust_cache('friends', self.from_user.pk)
 
@@ -121,6 +132,7 @@ class FriendshipRequest(models.Model):
         self.delete()
         friendship_request_canceled.send(sender=self)
         bust_cache('requests', self.to_user.pk)
+        bust_cache('sent_requests', self.from_user.pk)
         return True
 
     def mark_viewed(self):
@@ -140,7 +152,7 @@ class FriendshipManager(models.Manager):
         friends = cache.get(key)
 
         if friends is None:
-            qs = Friend.objects.select_related(depth=1).filter(to_user=user).all()
+            qs = Friend.objects.select_related('from_user', 'to_user').filter(to_user=user).all()
             friends = [u.from_user for u in qs]
             cache.set(key, friends)
 
@@ -152,7 +164,7 @@ class FriendshipManager(models.Manager):
         requests = cache.get(key)
 
         if requests is None:
-            qs = FriendshipRequest.objects.select_related(depth=1).filter(
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
                 to_user=user).all()
             requests = list(qs)
             cache.set(key, requests)
@@ -165,8 +177,8 @@ class FriendshipManager(models.Manager):
         requests = cache.get(key)
 
         if requests is None:
-            qs = FriendshipRequest.objects.select_related(depth=1).filter(
-                    from_user=user).all()
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
+                from_user=user).all()
             requests = list(qs)
             cache.set(key, requests)
 
@@ -178,7 +190,7 @@ class FriendshipManager(models.Manager):
         unread_requests = cache.get(key)
 
         if unread_requests is None:
-            qs = FriendshipRequest.objects.select_related(depth=1).filter(
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
                 to_user=user,
                 viewed__isnull=True).all()
             unread_requests = list(qs)
@@ -192,7 +204,7 @@ class FriendshipManager(models.Manager):
         count = cache.get(key)
 
         if count is None:
-            count = FriendshipRequest.objects.select_related(depth=1).filter(
+            count = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
                 to_user=user,
                 viewed__isnull=True).count()
             cache.set(key, count)
@@ -205,7 +217,7 @@ class FriendshipManager(models.Manager):
         read_requests = cache.get(key)
 
         if read_requests is None:
-            qs = FriendshipRequest.objects.select_related(depth=1).filter(
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
                 to_user=user,
                 viewed__isnull=False).all()
             read_requests = list(qs)
@@ -219,7 +231,7 @@ class FriendshipManager(models.Manager):
         rejected_requests = cache.get(key)
 
         if rejected_requests is None:
-            qs = FriendshipRequest.objects.select_related(depth=1).filter(
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
                 to_user=user,
                 rejected__isnull=False).all()
             rejected_requests = list(qs)
@@ -227,6 +239,33 @@ class FriendshipManager(models.Manager):
 
         return rejected_requests
 
+    def unrejected_requests(self, user):
+        """ All requests that haven't been rejected """
+        key = cache_key('unrejected_requests', user.pk)
+        unrejected_requests = cache.get(key)
+
+        if unrejected_requests is None:
+            qs = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
+                to_user=user,
+                rejected__isnull=True).all()
+            unrejected_requests = list(qs)
+            cache.set(key, unrejected_requests)
+
+        return unrejected_requests
+
+    def unrejected_request_count(self, user):
+        """ Return a count of unrejected friendship requests """
+        key = cache_key('unrejected_request_count', user.pk)
+        count = cache.get(key)
+
+        if count is None:
+            count = FriendshipRequest.objects.select_related('from_user', 'to_user').filter(
+                to_user=user,
+                rejected__isnull=True).count()
+            cache.set(key, count)
+
+        return count
+
     def add_friend(self, from_user, to_user):
         """ Create a friendship request """
         if from_user == to_user:
@@ -241,6 +280,7 @@ class FriendshipManager(models.Manager):
             raise AlreadyExistsError("Friendship already requested")
 
         bust_cache('requests', to_user.pk)
+        bust_cache('sent_requests', from_user.pk)
         friendship_request_created.send(sender=request)
 
         return request
